/**
 * 
 * All content copyright (c) 2003-2008 Terracotta, Inc.,
 * except as may otherwise be noted in a separate copyright notice.
 * All rights reserved.
 *
 */
package org.terracotta.message.channel;

import org.terracotta.message.pipe.Pipe;

public class DefaultChannel<T, ID> implements Channel<T, ID> 
{
	private final Pipe<T> sendPipe;
	private final Pipe<T> receivePipe;
	private ID routingID;
	
	public static class Factory<T, ID> implements Channel.Factory<T, ID> 
	{
		private final Pipe.Factory<T> pipeFactory;

		public Factory(final Pipe.Factory<T> pipeFactory) 
		{
			this.pipeFactory = pipeFactory;
		}

		public Channel<T, ID> create(ID routingID)
		{
			return new DefaultChannel<T, ID>(routingID, pipeFactory.create(), pipeFactory.create());
		}
	}

	/**
	 * Private constructor - use DefaultPipe.Factory.
	 * 
	 * @param sendPipe
	 * @param receivePipe
	 * @param allCompletedWork
	 */
	private DefaultChannel(final ID routingID, final Pipe<T> sendPipe, final Pipe<T> receivePipe) 
	{
		this.routingID = routingID;
		this.sendPipe = sendPipe;
		this.receivePipe = receivePipe;
	}

    public Pipe<T> getSendPipe() { return sendPipe; }
	public Pipe<T> getReceivePipe() { return receivePipe; }
	public ID getRoutingID() { return routingID; }
	
	public void clear()
	{
		sendPipe.clear();
		receivePipe.clear();
	}
}
