/**
 * 
 * All content copyright (c) 2003-2008 Terracotta, Inc.,
 * except as may otherwise be noted in a separate copyright notice.
 * All rights reserved.
 *
 */
package org.terracotta.message.pipe;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;

/**
 * An implementation of a Channel that uses a BlockingQueue.
 * 
 * @param <T>  The type of message passed in the Channel
 */
public class BlockingQueueBasedPipe<T> implements Pipe<T> 
{
    private final BlockingQueue<T> queue;

    public static class Factory<T> implements Pipe.Factory<T> 
	{
		private final int size;

		public Factory(final int size) { this.size = size; }
		public Pipe<T> create() {  return new BlockingQueueBasedPipe<T>(size); }
	}

	public BlockingQueueBasedPipe(int size) 
	{
		this(new LinkedBlockingQueue<T>(size));
	}

	public BlockingQueueBasedPipe(final BlockingQueue<T> queue) 
	{
	    this.queue = queue;
	}

	public T poll(long timeout, TimeUnit unit) throws InterruptedException
	{
		return queue.poll(timeout, unit);
	}

	public T put(T item) throws InterruptedException 
	{
		queue.put(item);
		return item;
	}

	public T take() throws InterruptedException 
	{
		return queue.take();
	}
	
	public T peek()
    {
	    return queue.peek();
    }

    public int size() {
		return queue.size();
	}

	public void clear() {
		queue.clear();
	}
}
