/**
 * 
 * All content copyright (c) 2003-2008 Terracotta, Inc.,
 * except as may otherwise be noted in a separate copyright notice.
 * All rights reserved.
 *
 */
package org.terracotta.message.pipe;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

public class ListBasedPipe<T> implements Pipe<T> {
	private final List<T> m_list;

	private final int m_maxSize;

	public static class Factory<T> implements Pipe.Factory<T> {
		private final int m_size;

		public Factory() {
			this(1000);
		}

		public Factory(final int size) {
			super();
			m_size = size;
		}

		public Pipe<T> create() {
			return new ListBasedPipe<T>(new ArrayList<T>(), m_size);
		}
	}

	public ListBasedPipe() {
		this(new ArrayList<T>(), 1000);
	}

	public ListBasedPipe(final List<T> list, int maxSize) {
		m_list = list;
		m_maxSize = maxSize;
	}

	public T put(T item) throws InterruptedException {
		synchronized (m_list) {
			while (m_list.size() >= m_maxSize) {
				m_list.wait();
			}

			m_list.add(item);
			m_list.notify();
		}

		return item;
	}

	public T take() throws InterruptedException {
		synchronized (m_list) {
			while (m_list.isEmpty()) {
				m_list.wait();
			}

			return m_list.remove(0);
		}
	}

	public T poll(long timeout, TimeUnit unit) throws InterruptedException 
	{
		long started = System.currentTimeMillis();
		synchronized (m_list) {
			while (m_list.isEmpty()) {
				m_list.wait(timeout - System.currentTimeMillis() + started);
				if (System.currentTimeMillis() - started >= timeout) {
					return null;
				}
			}

			return m_list.remove(0);
		}
	}

	public T peek()
    {
	    synchronized (m_list) {
	        if (m_list.size() == 0) { return null; }
	        return m_list.get(0);
	    }
    }

    public int size() 
    {
		synchronized (m_list) {
		    return m_list.size();
		}
	}

	public void clear() 
	{
	    synchronized (m_list) {
	        m_list.clear();
	    }
	}
}
