/**
 * 
 * All content copyright (c) 2003-2008 Terracotta, Inc.,
 * except as may otherwise be noted in a separate copyright notice.
 * All rights reserved.
 *
 */
package org.terracotta.message.pipe;


/**
 * Listens to a pipe, and calls the listener event method when something
 * arrives.
 * 
 * @param <T>
 */
public abstract class PipeListener<T, ID> implements Runnable 
{
	private static final String SHUTDOWN = "normal shutdown using shutdown()";
	private static final String STOP = "shutdown using stop()";
	
	protected final ID routingID;
	protected final Pipe<T> pipe;
	private volatile String reasonForShutdown = null;	
	private transient Thread listenThread;

	public PipeListener(final ID routingID, final Pipe<T> pipe) {
		this.routingID = routingID;
		this.pipe = pipe;
	}

	public abstract boolean event(T workItem) throws Exception;
	public void stopped() { }
	
	public void start() 
	{
		listenThread = new Thread(this);
		listenThread.setDaemon(true);
		listenThread.start();
	}

	public void stop() 
	{
	    reasonForShutdown = STOP;
		listenThread.interrupt();
	}

	public void shutdown() 
	{
        reasonForShutdown = SHUTDOWN;
        listenThread.interrupt();	    
	}
	
	public ID getRoutingID() 
	{
		return routingID;
	}
	
	public void run() 
	{
		while (reasonForShutdown != STOP && (reasonForShutdown != SHUTDOWN || pipe.peek() != null)) {
		    try {
		        final T kItem = pipe.take();
		        if (!event(kItem)) { break; }
		    } catch (InterruptedException ie) {
		        // normal shutdown
		    } catch (Exception e) {
		        reasonForShutdown = e.toString();
		        break;
		    }
		}
		stopped();
		System.out.println("pipe listener has been stopped due to: " + reasonForShutdown);
	}
}
