package org.terracotta.message.routing;

import java.util.List;

import org.terracotta.message.channel.Channel;
import org.terracotta.message.channel.ChannelManager;

public class RoundRobinRouter<T, ID> implements Router<T, ID>
{
    private int counter = 0;

    public Channel<T, ID> getRoute(ChannelManager<T, ID> channelManager)
    {
        int index;
        List<ID> routingIDs = channelManager.getRoutingIDs();
        if (routingIDs == null || routingIDs.size() == 0) { return null; }
        
        synchronized (this) {
            if (counter >= routingIDs.size()) {
                counter = 0;
            }
            index = counter++;
        }
        
        return channelManager.getOrCreateChannelFor(routingIDs.get(index));        
    }
}
