package org.terracotta.message.topology;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.terracotta.message.channel.ChannelManager;

public abstract class TopologyManager<T, ID>
{    
    public static final String DEFAULT_TOPOLOGY = "DEFAULT";
    
    private final Map<String, Topology<T, ID>> topologyMap = new ConcurrentHashMap<String, Topology<T, ID>>();
       
    protected abstract ChannelManager<T, ID> createChannelManager();
    
    public Topology<T, ID> getOrCreateTopology(String name)
    {
        synchronized (topologyMap) {
            if (topologyMap.containsKey(name)) {
                return topologyMap.get(name);
            }
            
            Topology<T, ID> topology = new Topology<T, ID>(name, createChannelManager());
            topologyMap.put(name, topology);
            return topology;
        }
    }
}
