package org.terracotta.workmanager;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.terracotta.message.channel.Channel;
import org.terracotta.message.channel.ChannelManager;
import org.terracotta.message.pipe.Pipe;
import org.terracotta.message.pipe.PipeListener;
import org.terracotta.message.topology.Topology;
import org.terracotta.message.topology.TopologyManager;
import org.terracotta.workmanager.worker.Worker;
import org.terracotta.workmanager.worker.WorkerTask;

import commonj.work.WorkException;

public class AbstractWorker implements Worker {
    public static final int WORKER_TIMEOUT_IN_SECONDS = 60;

    protected ExecutorService m_threadPool;
    protected ChannelManager<RoutableWorkItem<String>, String> m_channelManager;
    protected Channel<RoutableWorkItem<String>, String> m_channel;
    protected String m_routingID;
    protected IncomingWorkListener m_workListener;

    public AbstractWorker()
    {
        this(TopologyManager.DEFAULT_TOPOLOGY);
    }
    
    public AbstractWorker(String topology) {
        Topology<RoutableWorkItem<String>, String> t = DefaultTopologyManager.getInstance().getOrCreateTopology(topology);
        m_channelManager = t.getChannelManager();
    }

    public void start() throws WorkException {
        m_threadPool = Executors.newCachedThreadPool(); 
        m_channel = m_channelManager.getOrCreateChannelFor(m_routingID);
        m_workListener = new IncomingWorkListener(m_routingID, m_channel.getSendPipe());
        m_workListener.start();
        System.out.println("started worker with routing ID: " + m_routingID);
    }

    // TODO not thread safe
    public void stop() {
        m_workListener.stop();
        m_threadPool.shutdown();
        m_channelManager.removeChannelFor(m_routingID);
    }

    private class IncomingWorkListener extends PipeListener<RoutableWorkItem<String>, String> {
        public IncomingWorkListener(String routingID, Pipe<RoutableWorkItem<String>> pipe) {
            super(routingID, pipe);
        }

        public boolean event(RoutableWorkItem<String> workItem) throws Exception {
            m_threadPool.execute(new WorkerTask<RoutableWorkItem<String>>(workItem, m_channel.getReceivePipe()));
            return true;
        }
    }
}
