/**
 * 
 * All content copyright (c) 2003-2008 Terracotta, Inc.,
 * except as may otherwise be noted in a separate copyright notice.
 * All rights reserved.
 *
 */
package org.terracotta.workmanager.dynamic;

import org.terracotta.jmx.events.ClusterEvents;
import org.terracotta.message.routing.Router;
import org.terracotta.message.topology.TopologyManager;
import org.terracotta.workmanager.AbstractWorkManager;
import org.terracotta.workmanager.RoutableWorkItem;

/**
 * A routing aware WorkManager that uses an implementation of the Router
 * interface to do the route work to different work queues.
 */
public class DynamicWorkManager extends AbstractWorkManager {
	private ClusterEvents.DefaultListener m_eventsListener;

	public DynamicWorkManager(final Router<RoutableWorkItem<String>, String> router) {
		super(TopologyManager.DEFAULT_TOPOLOGY, router);
		m_eventsListener = new MyClusterEventsListener();
        ClusterEvents.registerListener(m_eventsListener);
        try { m_eventsListener.waitForRegistration(); } catch (InterruptedException ie) { } 
	}

    private class MyClusterEventsListener extends ClusterEvents.DefaultListener
    {
        private boolean isMasterNode(Object nodeId) {
            return nodeId.equals(m_eventsListener.getMyNodeId());
        }

        @Override
        public  void thisNodeId(final Object nodeId) {
            super.thisNodeId(nodeId);
            System.out.println("work manager node id: " + nodeId);
        }        
        
        @Override
        public void initialClusterMembers(Object[] nodeIds) {
            reset();
            
            String[] routingIDs = new String[nodeIds.length];
            for (int i = 0; i < nodeIds.length; i++) {
                routingIDs[i] = nodeIds[i].toString();
            }
            unregisterAllBut(routingIDs);
            
            for (Object nodeId : nodeIds) {
                if (isMasterNode(nodeId)) {             
                    return; // this is the master node
                }           
                register((String)nodeId);          
            }
        }

        @Override
        public void nodeConnected(Object nodeId) {
            if (isMasterNode(nodeId)) {
                return; // this is the master node
            }           
            register(nodeId.toString());
        }

        @Override
        public void nodeDisconnected(Object nodeId) {            
            // TODO: fail-over all pending work before unregister 
            unregister(nodeId.toString());            
        }
    }
}
